#!/bin/bash
#

# buildTBB.sh <product_dir> <e2|e4|e5> <debug|opt|prof> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e2|e4|e5|i1> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

select_compiler()
{
    if [[ "${basequal}" == "e5" ]]
    then
      compiler=gcc
      dotver=4.8.2
      ver=v4_8_2
      unset cqual
    elif [ "${basequal}" = "i1" ]
    then
      compiler=icc
      dotver=14.0.2
      ver=v14_0_2
      cqual=i1
    else
      echo "ERROR: unrecognized qualifer $basequal." 1>&2
      exit 1
    fi
}

setup_compiler()
{
    setup ${compiler} ${ver} ${cqual:+-q ${cqual}}
    fullver=$(${compiler} -dumpversion)
    if [ ${fullver} = ${dotver} ]
    then
      echo " OK - ${compiler} version is ${fullver}"
    else
      echo "ERROR: wrong ${compiler} version ${fullver}"
      exit 1
    fi
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=tbb
pkgver=v4_2_3
ssibuildshims_version=v0_06_09
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=tbb42_20140122oss

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

select_compiler

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [[ "${extraqual}" == "opt" ]] || [[ "${extraqual}" == "prof" ]]
then
    reltype="release"
elif [ "${extraqual}" = "debug" ]
then
    reltype="debug"
fi

setup_compiler

mkdir -p ${pkgdir}/lib
if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`
extra_command="cpp0x=1 -j ${ncore}"

if [[ "${compiler}" == icc ]]
then
  # Clean up variables set by icc setup for tbb.
  for var in $(printenv | grep tbb | grep -v PWD | cut -d= -f1); do
    eval ${var}=`dropit -D -p $(eval echo \\${\${var}}) tbb`
    if [[ -z "$(eval echo \\$\${var})" ]]; then
      eval unset ${var}
    fi
  done
  extra_command="${extra_command} offload=mic"
fi

set -x

# TBB requires an in-source build
cd ${pkgdir}
tar xf ${tardir}/${pkgtarname}_src.tgz || die "ERROR: untar source failed."

patch -b -p0 < ${patchdir}/tbb-${pkgdotver}.patch || die "ERROR: patch failed."

cd ${pkgdir}/${pkgtarname}/src

make compiler=${compiler} ${extra_command} tbb_${reltype} tbbmalloc_${reltype} || die "ERROR: make failed."

set +x

mv -v ../include "${pkgdir}"/

# Move things into the expected locations (must match UPS table file).
if [ ${OS1} = "Darwin" ]
then
  mv -v ${pkgdir}/${pkgtarname}/build/macos*/lib*.dylib ${pkgdir}/lib/
  ext=.dylib
else
  mv ${pkgdir}/${pkgtarname}/build/lin*/lib*.so* ${pkgdir}/lib/
  if [[ -d "${pkgdir}/${pkgtarname}/build/mic_icc_${reltype}" ]]; then
    mkdir ${pkgdir}/lib-mic
    mv -v "${pkgdir}/${pkgtarname}/build/mic_icc_${reltype}"/libtbb*.so* "${pkgdir}/lib-mic/"
  fi
  ext=.so
fi

mv -v "${pkgdir}/${pkgtarname}/doc" "${pkgdir}/"

# Convenience links for debug libraries.
if [ "${extraqual}" = "debug" ]; then
  make compiler=${compiler} ${extra_command} clean
  for dir in "${pkgdir}/lib" "${pkgdir}/lib-mic"; do
    [[ -d "$dir" ]] || continue
    ( cd "$dir"
      for i in *_debug${ext}; do
        ln -sv "${i}" "${i%_debug${ext}}${ext}"
      done )
  done
fi

# Source
cd "${pkgdir}/.."
[[ ! -d source ]] && mv "${pkgdir}/${pkgtarname}/src" source
rm -rf "${pkgdir}/${pkgtarname}"

# cmake configure files
cp -pr ${patchdir}/${package} ${pkgdir}/lib/

# ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "${package} is installed at ${TBB_FQ_DIR}"

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
